<?php
use WPPayForm\App\Services\GeneralSettings;

if (!defined('WPPAYFORM_VERSION')) {
	return;
}

if (!defined('PAYMATTIC_BAYARCASH_SLUG')) {
	define('PAYMATTIC_BAYARCASH_SLUG', 'paymattic_bayarcash');
}

$slug = PAYMATTIC_BAYARCASH_SLUG;

$paymattic_currency = GeneralSettings::getGlobalCurrencySettings();
$current_currency = $paymattic_currency['currency'] ?? '';

// Get the plugin directory URL for the logo path
$plugin_url = plugin_dir_url(dirname(__FILE__, 2));

if (!function_exists('paymattic_verify_bayarcash_token')) {
	function paymattic_verify_bayarcash_token($token, $is_sandbox = false): array {
		if (empty($token)) {
			return array(
				'status' => 'error',
				'message' => __('Token cannot be empty', 'bayarcash-for-paymattic')
			);
		}

		$api_url = $is_sandbox
			? 'https://console.bayarcash-sandbox.com/api/v2/portals'
			: 'https://console.bayar.cash/api/v2/portals';

		$response = wp_remote_get($api_url, array(
			'headers' => array(
				'Authorization' => 'Bearer ' . $token,
				'Accept' => 'application/json'
			),
			'timeout' => 15
		));

		if (is_wp_error($response)) {
			return array(
				'status' => 'error',
				'message' => __('Connection error: ', 'bayarcash-for-paymattic') . $response->get_error_message()
			);
		}

		$body = wp_remote_retrieve_body($response);
		$data = json_decode($body, true);

		if (!isset($data['data'])) {
			return array(
				'status' => 'error',
				'message' => __('Invalid token or API response', 'bayarcash-for-paymattic')
			);
		}

		return array(
			'status' => 'success',
			'message' => __('Token verified successfully', 'bayarcash-for-paymattic')
		);
	}
}

// Add AJAX handlers
add_action('wp_ajax_save_bayarcash_settings', function() {
	check_ajax_referer('bayarcash_settings', 'nonce');

	parse_str($_POST['settings'], $settings);
	update_option(PAYMATTIC_BAYARCASH_SLUG, $settings[PAYMATTIC_BAYARCASH_SLUG]);

	wp_send_json_success();
});

add_action('wp_ajax_verify_bayarcash_token', function() {
	check_ajax_referer('bayarcash_settings', 'nonce');

	$settings = get_option(PAYMATTIC_BAYARCASH_SLUG);
	$token = $_POST['token'] ?? '';
	$is_sandbox = filter_var($_POST['is_sandbox'] ?? false, FILTER_VALIDATE_BOOLEAN);

	$result = paymattic_verify_bayarcash_token($token, $is_sandbox);
	wp_send_json($result);
});

$global_configuration_fields = array(
	array(
		'type' => 'notice',
		'style' => 'danger',
		'content' => sprintf(__('The default currency is set to non compatible currencies! %sClick here%s to update currency configuration.', 'bayarcash-for-paymattic'), '<a target=_blank href=' . admin_url('admin.php?page=wppayform_settings#/payment') . ' >', '</a>'),
		'class' => $current_currency == 'MYR' ? 'hidden' : '',
	),
	array(
		'type'    => 'subheading',
		'content' => 'Credentials',
	),
	array(
		'type'    => 'content',
		'content' => sprintf(__('Get your credentials from %sBayarcash Console Profile%s', 'bayarcash-for-paymattic'), '<a href="https://console.bayar.cash/profile" target="_blank">', '</a>'),
	),
	array(
		'id'      => 'enable-sandbox',
		'type'    => 'switcher',
		'title'   => __('Enable Sandbox Mode', 'bayarcash-for-paymattic'),
		'desc'    => __('Toggle this to switch between sandbox and production environments.', 'bayarcash-for-paymattic'),
		'help'    => __('Use sandbox mode to test payments without processing real transactions. Make sure to use sandbox credentials when this is enabled.', 'bayarcash-for-paymattic'),
		'default' => false,
		'class'   => 'bc-sandbox-switcher'
	),
	array(
		'id'         => 'sandbox-notice',
		'type'       => 'notice',
		'style'      => 'warning',
		'content'    => __('Sandbox mode is enabled. Make sure to use sandbox credentials for testing.', 'bayarcash-for-paymattic'),
		'dependency' => array('enable-sandbox', '==', 'true'),
	),
	array(
		'id'    => 'personal-access-token',
		'type'  => 'textarea',
		'title' => __('Personal Access Token', 'bayarcash-for-paymattic'),
		'desc'  => sprintf(__('Enter your Personal Access Token. Get it from %sBayarcash Console Profile%s', 'bayarcash-for-paymattic'), '<a href="https://console.bayar.cash/profile" target="_blank">', '</a>'),
		'help'  => __('Personal Access Token is used to authenticate your requests to the Bayarcash API.', 'bayarcash-for-paymattic'),
		'after' => '
	    <div class="bc-token-status" style="margin-top: 10px;">
	        <span class="status-message"></span>
	        <button type="button" class="button button-secondary verify-token" style="margin-left: 10px;">
	            ' . __('Verify Token', 'bayarcash-for-paymattic') . '
	        </button>
	    </div>'
	),
	array(
		'id'    => 'portal-key',
		'type'  => 'text',
		'title' => __('Portal Key', 'bayarcash-for-paymattic'),
		'desc'  => __('Enter your Portal Key.', 'bayarcash-for-paymattic'),
		'help'  => __('Portal Key is required for accessing the Bayarcash portal features.', 'bayarcash-for-paymattic'),
	),
	array(
		'id'    => 'api-secret-key',
		'type'  => 'text',
		'title' => __('API Secret Key', 'bayarcash-for-paymattic'),
		'desc'  => sprintf(__('Enter your API Secret Key. Get it from %sBayarcash Console Profile%s', 'bayarcash-for-paymattic'), '<a href="https://console.bayar.cash/profile" target="_blank">', '</a>'),
		'help'  => __('API Secret Key is used to secure your API requests to Bayarcash.', 'bayarcash-for-paymattic'),
	),
	array(
		'type'    => 'subheading',
		'content' => 'Miscellaneous',
	),
	array(
		'id'          => 'payment-title',
		'type'        => 'text',
		'title'       => __('Payment Title', 'bayarcash-for-paymattic'),
		'desc'        => __('Enter your Payment Title. Default is <strong>Bayarcash</strong>', 'bayarcash-for-paymattic'),
		'help'        => __('This allows you to customize the payment title.', 'bayarcash-for-paymattic'),
		'placeholder' => 'Bayarcash',
		'default'     => 'Bayarcash',
	),
);

CSF_Setup::createOptions($slug, array(
	'framework_title' => sprintf(
		'<img src="%sassets/bayarcash.png" style="max-height: 40px; margin-right: 5px; vertical-align: middle;" /> %s',
		$plugin_url,
		__('for Paymattic', 'bayarcash-for-paymattic')
	),
	'menu_title'  => __('Bayarcash Settings', 'bayarcash-for-paymattic'),
	'menu_slug'   => 'bayarcash-for-paymattic',
	'menu_type'   => 'submenu',
	'menu_parent' => 'wppayform.php',
	'footer_text' => sprintf(__('Bayarcash for Paymattic %s', 'bayarcash-for-paymattic'), PAYMATTIC_BAYARCASH_MODULE_VERSION),
	'theme'       => 'light',
));

CSF_Setup::createSection($slug, array(
	'id'          => 'global-configuration',
	'title'       => __('Global Configuration', 'bayarcash-for-paymattic'),
	'icon'        => 'fa fa-home',
	'fields'      => $global_configuration_fields
));