/**
 * Bayarcash Admin JavaScript
 * Handles token verification for both global and form-specific settings
 */

(function($) {
    'use strict';

    // Token verification function for global settings
    function verifyBayarcashToken() {
        handleTokenVerification({
            tokenSelector: "textarea[data-depend-id='personal-access-token']",
            sandboxSelector: ".bc-sandbox-switcher .csf--switcher",
            statusSelector: ".bc-token-status .status-message"
        });
    }

    // Token verification function for specific forms
    function verifyBayarcashTokenForm(formId) {
        handleTokenVerification({
            tokenSelector: `textarea[data-depend-id="personal-access-token-${formId}"]`,
            sandboxSelector: `.bc-sandbox-switcher-${formId} .csf--switcher`,
            statusSelector: `.bc-token-status-${formId} .status-message`,
            formId: formId
        });
    }

    // Common token verification handler
    function handleTokenVerification(config) {
        const token = document.querySelector(config.tokenSelector)?.value;
        const sandboxSwitch = document.querySelector(config.sandboxSelector);
        const isSandbox = sandboxSwitch ? sandboxSwitch.classList.contains("csf--active") : false;
        const statusMessage = document.querySelector(config.statusSelector);

        if (!statusMessage) {
            console.error('Status message element not found');
            return;
        }

        if (!token) {
            updateStatus(statusMessage, bayarcashAdmin.i18n.tokenEmpty, 'error');
            return;
        }

        updateStatus(statusMessage, bayarcashAdmin.i18n.verifying, '');

        $.ajax({
            url: bayarcashAdmin.ajaxUrl,
            type: "POST",
            data: {
                action: "verify_bayarcash_token",
                token: token,
                is_sandbox: isSandbox,
                form_id: config.formId || null,
                nonce: bayarcashAdmin.nonce
            },
            success: function(response) {
                updateStatus(statusMessage, response.message, response.status);
            },
            error: function() {
                updateStatus(statusMessage, bayarcashAdmin.i18n.verificationFailed, 'error');
            }
        });
    }

    // Helper function to update status message
    function updateStatus(element, message, status) {
        element.innerHTML = message;
        element.classList.remove('success', 'error');
        if (status) {
            element.classList.add(status);
        }
    }

    // Setup mutation observer for sandbox switch
    function setupSandboxObserver(config) {
        const sandboxSwitch = document.querySelector(config.sandboxSelector);
        if (!sandboxSwitch) return;

        const observer = new MutationObserver(function(mutations) {
            mutations.forEach(function(mutation) {
                if (mutation.attributeName === "class") {
                    const token = document.querySelector(config.tokenSelector);
                    if (token?.value) {
                        config.verifyFn();
                    }
                }
            });
        });

        observer.observe(sandboxSwitch, {
            attributes: true
        });
    }

    // Initialize when DOM is ready
    $(document).ready(function() {
        // Add styles for status messages
        const style = document.createElement('style');
        style.textContent = `
            .bc-token-status .success,
            [class^="bc-token-status-"] .success { 
                color: green; 
            }
            .bc-token-status .error,
            [class^="bc-token-status-"] .error { 
                color: red; 
            }
        `;
        document.head.appendChild(style);

        // Handle global settings
        const globalToken = document.querySelector("textarea[data-depend-id='personal-access-token']");
        if (globalToken?.value) {
            verifyBayarcashToken();
        }
        setupSandboxObserver({
            sandboxSelector: ".bc-sandbox-switcher .csf--switcher",
            tokenSelector: "textarea[data-depend-id='personal-access-token']",
            verifyFn: verifyBayarcashToken
        });

        // Handle form-specific settings
        const formFields = document.querySelectorAll('[data-depend-id^="personal-access-token-"]');
        formFields.forEach(field => {
            const formId = field.getAttribute('data-depend-id').replace('personal-access-token-', '');
            if (field.value) {
                verifyBayarcashTokenForm(formId);
            }
            setupSandboxObserver({
                sandboxSelector: `.bc-sandbox-switcher-${formId} .csf--switcher`,
                tokenSelector: `textarea[data-depend-id="personal-access-token-${formId}"]`,
                verifyFn: () => verifyBayarcashTokenForm(formId)
            });
        });

        // Bind click handlers
        $(document).on('click', '.verify-token', function() {
            const formId = this.closest('[class^="bc-token-status-"]')?.className.match(/bc-token-status-(\d+)/)?.[1];
            if (formId) {
                verifyBayarcashTokenForm(formId);
            } else {
                verifyBayarcashToken();
            }
        });
    });

    // Expose necessary functions globally
    window.verifyBayarcashToken = verifyBayarcashToken;
    window.verifyBayarcashTokenForm = verifyBayarcashTokenForm;

})(jQuery);