/**
 * Bayarcash Gateway Fee Handler for Paymattic
 */
(function($) {
    'use strict';

    class BayarcashPaymattic {
        constructor() {
            this.feeSettings = window.bayarcashPaymattic?.feeSettings || {};
            this.isUpdating = false;
            this.init();
        }

        init() {
            $(document).ready(() => {
                this.initializeForm();
                this.bindEvents();
            });
        }

        bindEvents() {
            // Listen for payment method changes
            $(document).on('change', 'input[name="__wpf_selected_payment_method"]', () => {
                this.updateTotals();
            });

            // Watch for changes in payment amounts
            $(document).on('change', '[id^="payment_item_price"]', () => {
                this.updateTotals();
            });
        }

        initializeForm() {
            const $table = $('#wpf_table_div table');
            if ($table.length) {
                this.updateTotals();
            }
        }

        getCurrentPaymentMethod() {
            return $('input[name="__wpf_selected_payment_method"]:checked').val();
        }

        parseAmount(amountText) {
            if (!amountText) return 0;
            // Remove 'RM' prefix, any commas, and spaces, then parse as float
            return parseFloat(amountText.replace('RM', '').replace(/,/g, '').trim()) || 0;
        }

        formatNumber(number) {
            return number.toFixed(2).replace(/\B(?=(\d{3})+(?!\d))/g, ",");
        }

        calculateOriginalTotal() {
            const $table = $('#wpf_table_div table');
            if (!$table.length) return 0;

            let originalTotal = 0;
            $table.find('tbody tr').each((_, row) => {
                const $row = $(row);
                // Skip if this is the total row or a fee row
                if ($row.attr('id') === 'wpf_form_id_26_total' || $row.hasClass('gateway-fee-row')) {
                    return;
                }
                const priceText = $row.find('#payment_item_price').text();
                const qty = parseInt($row.find('#payment_item_qty').text()) || 1;
                const price = this.parseAmount(priceText);
                originalTotal += price * qty;
            });

            return originalTotal;
        }

        getFeeConfig(paymentMethod) {
            const ownSettingMethods = ['fpx_banking', 'credit_card', 'duitnow_banking', 'duitnow_qr'];
            const duitnowQRMethods = [
                'spaylater',
                'boost_payflex',
                'indo_banking',
                'indo_ewallet',
                'nets_sg'
            ];

            const effectiveMethod = ownSettingMethods.includes(paymentMethod)
                ? paymentMethod
                : (duitnowQRMethods.includes(paymentMethod) ? 'duitnow_qr' : paymentMethod);

            const methodSettings = this.feeSettings[effectiveMethod] || {};
            const isEnabled = methodSettings[`${effectiveMethod}_fee_enable`] === 'true';

            if (!isEnabled) return null;

            return {
                enabled: true,
                feeType: methodSettings[`${effectiveMethod}_fee_type`] || 'flat',
                flatFee: parseFloat(methodSettings[`${effectiveMethod}_flat_fee`]) || 0,
                percentageFee: parseFloat(methodSettings[`${effectiveMethod}_percentage_fee`]) || 0,
                description: methodSettings[`${effectiveMethod}_fee_description`] || 'Processing Fee'
            };
        }

        calculateFee(amount, paymentMethod) {
            const config = this.getFeeConfig(paymentMethod);
            if (!config) return 0;

            let fee = 0;
            switch (config.feeType) {
                case 'flat':
                    fee = config.flatFee;
                    break;
                case 'percentage':
                    fee = amount * (config.percentageFee / 100);
                    break;
                case 'both':
                    fee = config.flatFee + (amount * (config.percentageFee / 100));
                    break;
            }

            return parseFloat(fee.toFixed(2));
        }

        getFeeDescription(paymentMethod) {
            const config = this.getFeeConfig(paymentMethod);
            if (!config) return '';

            let description = config.description;

            if (config.feeType === 'percentage') {
                const percentage = config.percentageFee % 1 === 0 ?
                    Math.floor(config.percentageFee) :
                    config.percentageFee.toFixed(1);
                description += ` (${percentage}%)`;
            } else if (config.feeType === 'both') {
                const percentage = config.percentageFee % 1 === 0 ?
                    Math.floor(config.percentageFee) :
                    config.percentageFee.toFixed(1);
                description += ` (RM${config.flatFee.toFixed(2)} + ${percentage}%)`;
            } else {
                description += ` (RM${config.flatFee.toFixed(2)})`;
            }

            return description;
        }

        updateTotals() {
            if (this.isUpdating) return;

            this.isUpdating = true;
            try {
                const $table = $('#wpf_table_div table');
                if (!$table.length) return;

                const originalTotal = this.calculateOriginalTotal();
                const paymentMethod = this.getCurrentPaymentMethod();
                const gatewayFee = paymentMethod ? this.calculateFee(originalTotal, paymentMethod) : 0;
                const total = originalTotal + gatewayFee;

                // Remove existing fee row if exists
                $table.find('tr.gateway-fee-row').remove();

                // Add fee row if applicable
                if (paymentMethod && gatewayFee > 0) {
                    const feeDescription = this.getFeeDescription(paymentMethod);
                    const $totalRow = $table.find('#wpf_form_id_26_total');
                    const $feeRow = $(`
                        <tr class="gateway-fee-row">
                            <td>${feeDescription}</td>
                            <td>RM${this.formatNumber(gatewayFee)}</td>
                            <td>1</td>
                            <td>RM${this.formatNumber(gatewayFee)}</td>
                        </tr>
                    `);
                    $totalRow.before($feeRow);
                }

                // Update total row
                const $totalCell = $table.find('#wpf_form_id_26_total strong');
                if ($totalCell.length) {
                    $totalCell.text(`RM${this.formatNumber(total)}`);
                }

                // Update hidden input for form submission if exists
                $('[name="payment_total"]').val(total.toFixed(2));

                // Update the payment button total amount
                $('.wpf_calc_payment_total').text(`RM${this.formatNumber(total)}`);

            } catch (error) {
                console.error('Update totals error:', error);
            } finally {
                this.isUpdating = false;
            }
        }
    }

    // Initialize when document is ready
    $(document).ready(() => {
        window.bayarcashPaymatticHandler = new BayarcashPaymattic();
    });

})(jQuery);