<?php

namespace BayarcashPaymattic;

use WPPayForm\App\Modules\FormComponents\BaseComponent;
use WPPayForm\Framework\Support\Arr;
use WPPayForm\App\Models\Form;

if (!defined('ABSPATH')) {
	exit;
}

class PaymentChannelElement extends BaseComponent
{
	private static $_instances = [];
	private $channel;
	private array $channel_config;

	private static array $available_channels = [
		'fpx_banking' => [
			'title' => 'FPX Online Banking',
			'text_domain' => 'fpx-banking-paymattic'
		],
		'credit_card' => [
			'title' => 'Credit Card',
			'text_domain' => 'credit-card-paymattic'
		],
		'duitnow_banking' => [
			'title' => 'DuitNow Online Banking',
			'text_domain' => 'duitnow-banking-paymattic'
		],
		'duitnow_qr' => [
			'title' => 'DuitNow QR',
			'text_domain' => 'duitnow-qr-paymattic'
		],
		'spaylater' => [
			'title' => 'SPayLater',
			'text_domain' => 'spaylater-paymattic'
		],
		'boost_payflex' => [
			'title' => 'Boost PayFlex',
			'text_domain' => 'boost-payflex-paymattic'
		],
		'indo_banking' => [
			'title' => 'Indonesia Online Banking',
			'text_domain' => 'indo-banking-paymattic'
		],
		'indo_ewallet' => [
			'title' => 'Indonesia e-Wallet',
			'text_domain' => 'indo-ewallet-paymattic'
		],
		'nets_sg' => [
			'title' => 'NETS Singapore',
			'text_domain' => 'nets-sg-paymattic'
		]
	];

	public static function get_instance($channel)
	{
		if (!isset(self::$_instances[$channel])) {
			self::$_instances[$channel] = new self($channel);
		}
		return self::$_instances[$channel];
	}

	public static function init_all_channels()
	{
		foreach (array_keys(self::$available_channels) as $channel) {
			self::get_instance($channel);
		}
	}

	private function __construct($channel)
	{
		if (!isset(self::$available_channels[$channel])) {
			throw new \InvalidArgumentException("Invalid payment channel: $channel");
		}

		$this->channel = $channel;
		$this->channel_config = self::$available_channels[$channel];

		parent::__construct("{$channel}_element", 27);

		// Add filters and actions for this channel
		add_filter("wppayform/validate_gateway_api_{$channel}", function ($data, $form) {
			return $this->validate_api($form->ID);
		}, 2, 10);

		add_action("wppayform/payment_method_choose_element_render_{$channel}",
			array($this, 'renderForMultiple'), 10, 3);

		add_filter('wppayform/available_payment_methods',
			array($this, 'push_payment_method'), 2, 1);
	}

	public function component(): array
	{
		return array(
			'type' => "{$this->channel}_element",
			'editor_title' => __($this->channel_config['title'], $this->channel_config['text_domain']),
			'conditional_hide' => true,
			'editor_icon' => '',
			'group' => 'payment_method_element',
			'method_handler' => $this->channel,
			'position_group' => 'payment_method',
			'single_only' => true,
			'editor_elements' => array(
				'label' => array(
					'label' => 'Field Label',
					'type' => 'text'
				)
			),
			'field_options' => array(
				'label' => __($this->channel_config['title'], $this->channel_config['text_domain'])
			)
		);
	}

	public function render($element, $form, $elements)
	{
		$validation = $this->validate_api($form->ID);

		if (!$validation['is_valid']) {
			$missing = implode(', ', $validation['missing_fields']);
			?>
            <p style="color: red">
                Please configure payment gateway by setting the following API credentials:
                <b><?php echo esc_html($missing); ?></b>
                from <b>Paymattic Pro->Bayarcash Settings</b> to start accepting payments
            </p>
			<?php
			return;
		}

		if (!$this->is_supported_currency($form->ID)) {
			?>
            <p style="color: red">
                This payment method only supports MYR (Malaysian Ringgit)!<br/>
                Set appropriate currency from <b>Paymattic->Settings->Currency</b> to start accepting payments!
            </p>
			<?php
			return;
		}

		echo '<input data-wpf_payment_method="' . esc_attr($this->channel) . '" type="hidden" name="__' . esc_attr($this->channel) . '_payment_gateway" value="' . esc_attr($this->channel) . '" />';
	}

	private function validate_api($form_id): array
	{
		$result = [
			'is_valid' => true,
			'missing_fields' => []
		];

		// Check if options exist
		if (!($options = get_option(PAYMATTIC_BAYARCASH_SLUG))) {
			$result['is_valid'] = false;
			$result['missing_fields'][] = 'bayarcash_settings';
			return $result;
		}

		$postfix = '';
		if (!empty($options['form-customize-' . $form_id])) {
			$postfix = "-$form_id";
		}

		// Required API fields to check
		$required_fields = [
			'personal-access-token' . $postfix => 'Personal Access Token',
			'portal-key' . $postfix => 'Portal Key',
			'api-secret-key' . $postfix => 'API Secret Key'
		];

		// Validate each required field
		foreach ($required_fields as $field_key => $field_name) {
			if (empty($options[$field_key]) || strlen(trim($options[$field_key])) < 1) {
				$result['is_valid'] = false;
				$result['missing_fields'][] = $field_name;
			}
		}

		return $result;
	}

	private function is_supported_currency($form_id): bool
	{
		$currency_setting = Form::getCurrencySettings($form_id);
		return Arr::get($currency_setting, 'currency') === 'MYR';
	}

	public function renderForMultiple($paymentSettings, $form, $elements)
	{
		$component = $this->component();
		$component['id'] = "{$this->channel}_element";
		$component['field_options'] = $paymentSettings;
		$this->render($component, $form, $elements);
	}

	public function push_payment_method($methods)
	{
		$options = get_option(PAYMATTIC_BAYARCASH_SLUG);
		if (Arr::get($options, "enable_{$this->channel}", 'yes') === 'yes') {
			$methods[$this->channel] = array(
				'label' => __($this->channel_config['title'], $this->channel_config['text_domain']),
				'isActive' => true,
				'logo' => WPPAYFORM_URL . 'assets/images/gateways/billplz.svg',
				'editor_elements' => array(
					'label' => array(
						'label' => __('Payment Option Label', $this->channel_config['text_domain']),
						'type' => 'text',
						'default' => 'Pay with ' . $this->channel_config['title']
					)
				)
			);
		}

		return $methods;
	}
}