<?php

namespace BayarcashPaymattic;

use Exception;

class Plugin {
	private static $_instance;

	public static function get_instance(): self {
		if (self::$_instance == null) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct() {
		$this->define();
		$this->includes();
		$this->add_filters();
		$this->add_scripts();

		// Initialize payment channels
		Settings::init_all_channels();
		PaymentChannelElement::init_all_channels();
		Processor::get_instance();

		add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_assets'));
	}

	public function enqueue_admin_assets(): void {
		// Get current page
		$current_page = $_GET['page'] ?? '';

		// Load CSS only on WPPayForm page
		if ($current_page === 'wppayform.php') {
			wp_enqueue_style(
				'bayarcash-admin',
				PAYMATTIC_BAYARCASH_URL . 'includes/css/admin-style.css',
				array(),
				PAYMATTIC_BAYARCASH_MODULE_VERSION
			);
		}

		// Load JS only on Bayarcash settings page
		if ($current_page === 'bayarcash-for-paymattic') {
			wp_enqueue_script(
				'bayarcash-admin',
				PAYMATTIC_BAYARCASH_URL . 'includes/js/admin.js',
				array('jquery'),
				PAYMATTIC_BAYARCASH_MODULE_VERSION,
				true
			);

			wp_localize_script(
				'bayarcash-admin',
				'bayarcashAdmin',
				array(
					'ajaxUrl' => admin_url('admin-ajax.php'),
					'nonce' => wp_create_nonce('bayarcash_settings'),
					'i18n' => array(
						'tokenEmpty' => __('Token cannot be empty', 'bayarcash-for-paymattic'),
						'verifying' => __('Verifying...', 'bayarcash-for-paymattic'),
						'verificationFailed' => __('Verification failed', 'bayarcash-for-paymattic')
					)
				)
			);
		}
	}

	private function add_scripts(): void {
		add_action('wp_enqueue_scripts', function() {
			$js_url = PAYMATTIC_BAYARCASH_URL . 'includes/js/bayarcash-paymattic.js';

			try {
				wp_enqueue_script(
					'bayarcash-paymattic',
					$js_url,
					['jquery'],
					PAYMATTIC_BAYARCASH_MODULE_VERSION,
					true
				);

				// Get fee settings from CodeStar Framework
				$fee_settings = get_option(PAYMATTIC_BAYARCASH_SLUG, []);

				// Transform the settings to match JavaScript expectations
				$transformed_settings = [];
				$payment_methods = ['fpx_banking', 'credit_card', 'duitnow_banking', 'duitnow_qr'];

				foreach ($payment_methods as $method) {
					$transformed_settings[$method] = [
						"{$method}_fee_enable" => !empty($fee_settings["{$method}_fee_enable"]) ? 'true' : 'false',
						"{$method}_fee_type" => $fee_settings["{$method}_fee_type"] ?? 'flat',
						"{$method}_flat_fee" => $fee_settings["{$method}_flat_fee"] ?? '0',
						"{$method}_percentage_fee" => $fee_settings["{$method}_percentage_fee"] ?? '0',
						"{$method}_fee_description" => $fee_settings["{$method}_fee_description"] ?? ''
					];
				}

				// Add localized data
				wp_localize_script('bayarcash-paymattic', 'bayarcashPaymattic', [
					'ajaxUrl' => admin_url('admin-ajax.php'),
					'nonce' => wp_create_nonce('bayarcash_paymattic_nonce'),
					'feeSettings' => $transformed_settings
				]);

			} catch (Exception $e) {
				error_log('Bayarcash Paymattic: Error enqueueing JavaScript: ' . $e->getMessage());
			}
		});
	}

	private function define(): void {
		define('PAYMATTIC_BAYARCASH_FILE', __FILE__);
		define('PAYMATTIC_BAYARCASH_BASENAME', plugin_basename(PAYMATTIC_BAYARCASH_FILE));
		define('PAYMATTIC_BAYARCASH_DIR_PATH', plugin_dir_path(PAYMATTIC_BAYARCASH_FILE));
		define('PAYMATTIC_BAYARCASH_URL', plugin_dir_url(PAYMATTIC_BAYARCASH_FILE));
		define('PAYMATTIC_BAYARCASH_SLUG', 'paymattic_bayarcash');
	}

	private function includes(): void {
		// Admin-only includes
		if (is_admin()) {
			require_once PAYMATTIC_BAYARCASH_DIR_PATH . 'includes/admin/global-settings.php';
			require_once PAYMATTIC_BAYARCASH_DIR_PATH . 'includes/admin/form-settings.php';
			require_once PAYMATTIC_BAYARCASH_DIR_PATH . 'includes/admin/gateway-fee.php';
			require_once PAYMATTIC_BAYARCASH_DIR_PATH . 'includes/admin/backup-settings.php';
		}
	}

	public function setting_link($links): array {
		$new_links = array(
			'settings' => sprintf(
				'<a href="%1$s">%2$s</a>',
				admin_url('admin.php?page=bayarcash-for-paymattic'),
				esc_html__('Settings', 'bayarcash-for-paymattic')
			)
		);
		return array_merge($new_links, $links);
	}

	private function add_filters(): void {
		add_filter('plugin_action_links_' . PAYMATTIC_BAYARCASH_BASENAME, array($this, 'setting_link'));
	}
}