<?php

namespace BayarcashPaymattic;

use WPPayForm\Framework\Support\Arr;
use WPPayForm\App\Models\Transaction;
use WPPayForm\App\Models\Submission;
use WPPayForm\App\Services\ConfirmationHelper;
use WPPayForm\App\Models\Form;
use WPPayForm\App\Services\PlaceholderParser;
use Webimpian\BayarcashSdk\Bayarcash;
use Exception;

if (!defined('ABSPATH')) {
	exit;
}

class Processor{
	private static $_instance;
	private array $supported_currencies = ['MYR'];
	private array $payment_methods = [
		'fpx_banking' => [
			'id' => 1,
			'title' => 'FPX Online Banking'
		],
		'credit_card' => [
			'id' => 4,
			'title' => 'Credit/Debit Card'
		],
		'duitnow_banking' => [
			'id' => 5,
			'title' => 'DuitNow Online Banking'
		],
		'duitnow_qr' => [
			'id' => 6,
			'title' => 'DuitNow QR'
		],
		'spaylater' => [
			'id' => 7,
			'title' => 'ShopeePayLater'
		],
		'boost_payflex' => [
			'id' => 8,
			'title' => 'Boost PayFlex'
		],
		'indo_banking' => [
			'id' => 9,
			'title' => 'Indonesia Online Banking'
		],
		'indo_ewallet' => [
			'id' => 10,
			'title' => 'Indonesia E-Wallet'
		],
		'nets_sg' => [
			'id' => 11,
			'title' => 'NETS Singapore'
		]
	];

	public static function get_instance() {
		if (self::$_instance == null) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct() {
		$this->add_filters();
		$this->add_actions();
	}

	private function add_filters() {
		add_filter('wppayform/choose_payment_method_for_submission', array($this, 'choose_payment_method'), 10, 4);
		add_filter('wppayform/entry_transactions_bayarcash', array($this, 'add_transaction_url'), 10, 2);
		add_filter('wppayform/submitted_payment_items_bayarcash', array($this, 'validate_subscription'), 10, 4);
	}

	private function add_actions() {
		foreach (array_keys($this->payment_methods) as $method) {
			add_action("wppayform/form_submission_make_payment_{$method}", array($this, 'make_form_payment'), 10, 6);
			add_action("wppayform_payment_frameless_{$method}", array($this, 'redirect'));
			add_action("wpf_ipn_endpoint_{$method}", array($this, 'callback'));
		}
	}

	public function choose_payment_method($payment_method, $elements, $form_id, $form_data) {
		if ($payment_method) {
			return $payment_method;
		}

		foreach ($elements as $element) {
			foreach (array_keys($this->payment_methods) as $channel) {
				if (isset($element['type']) && $element['type'] === "{$channel}_element") {
					$submitted_method = Arr::get($form_data, "__${channel}_payment_gateway");
					if ($submitted_method && $submitted_method === $channel) {
						return $channel;
					}
				}
			}
		}

		return $payment_method;
	}

	public function make_form_payment($transaction_id, $submission_id, $form_data, $form, $has_subscriptions) {
		$transaction_model = new Transaction();
		$transaction = $transaction_model->getTransaction($transaction_id);
		$submission = (new Submission())->getSubmission($submission_id);

		$payment_method = $this->determine_payment_method($form_data);
		if (!$payment_method) {
			wp_send_json_error([
				'message' => __('Payment method not specified', 'bayarcash-for-paymattic')
			], 400);
			return;
		}

		$this->is_form_currency_supported(strtoupper($transaction->currency));
		$this->handle_purchase($transaction, $submission, $form_data, $form, $payment_method);
	}

	private function get_fee_method( $payment_method ) {
		$direct_methods = [
			'fpx_banking',
			'credit_card',
			'duitnow_banking'
		];

		return in_array( $payment_method, $direct_methods ) ? $payment_method : 'duitnow_qr';
	}

	private function calculate_gateway_fee($payment_method, $amount, $option) {
		$fee_settings = get_option(PAYMATTIC_BAYARCASH_SLUG);
		$fee_method = $this->get_fee_method($payment_method);
		$fee_enable_key = "{$fee_method}_fee_enable";

		// Check if fees are enabled for this payment method
		if (empty($fee_settings[$fee_enable_key])) {
			return 0;
		}

		$fee_type = $fee_settings["{$fee_method}_fee_type"];
		$flat_fee = floatval($fee_settings["{$fee_method}_flat_fee"]);
		$percentage_fee = floatval($fee_settings["{$fee_method}_percentage_fee"]);

		$total_fee = 0;

		switch ($fee_type) {
			case 'flat':
				$total_fee = $flat_fee;
				break;
			case 'percentage':
				$total_fee = ($amount * $percentage_fee) / 100;
				break;
			case 'both':
				$total_fee = $flat_fee + (($amount * $percentage_fee) / 100);
				break;
		}

		return $total_fee;
	}
	private function handle_purchase($transaction, $submission, $form_data, $form, $payment_method) {
		try {
			// Validate payment method
			if (!isset($this->payment_methods[$payment_method])) {
				throw new \Exception(__('Invalid payment method', 'bayarcash-for-paymattic'));
			}

			// Get form settings
			$option = $this->get_settings($form->ID);

			// Initialize Bayarcash SDK
			$bayarcash = $this->initialize_bayarcash($option);

			// Prepare submission data
			$submission_model = new Submission();
			$entries = $submission_model->getParsedSubmission($submission);

			// Calculate base amount (original payment total divided by 100)
			$base_amount = $transaction->payment_total / 100;

			// Calculate gateway fee
			$gateway_fee = $this->calculate_gateway_fee($payment_method, $base_amount, $option);

			// Add gateway fee to total amount
			$total_amount = $base_amount + $gateway_fee;

			// Build URLs
			$return_url = add_query_arg([
				'wppayform_payment' => $submission->id,
				'payment_method' => $payment_method,
				'submission_hash' => $submission->submission_hash,
				'gateway_return' => 1
			], site_url('index.php'));

			$callback_url = add_query_arg([
				'wpf_payment_api_notify' => 1,
				'payment_method' => $payment_method,
				'submission_id' => $submission->id
			], site_url('index.php'));

			// Prepare payment data
			$paymentData = [
				'portal_key' => $option['portal_key'],
				'payment_channel' => $this->payment_methods[$payment_method]['id'],
				'order_number' => $submission->id,
				'amount' => $total_amount, // Using the total amount including gateway fee
				'payer_name' => $this->get_customer_name($entries),
				'payer_email' => $this->get_customer_email($entries),
				'payer_telephone_number' => $this->get_customer_phone($entries),
				'return_url' => $return_url,
				'callback_url' => $callback_url,
				'platform_id' => 'paymattic'
			];

			// Add gateway fee information to payment note if fee was applied
			if ($gateway_fee > 0) {
				$fee_settings = get_option(PAYMATTIC_BAYARCASH_SLUG);
				$fee_method = $this->get_fee_method($payment_method);
				$fee_description = $fee_settings["{$fee_method}_fee_description"];

				// Update transaction with fee information
				$transaction_model = new Transaction();
				$transaction_model->updateTransaction($transaction->id, [
					'payment_note' => maybe_serialize([
						'base_amount' => $base_amount,
						'gateway_fee' => $gateway_fee,
						'fee_description' => $fee_description,
						'total_amount' => $total_amount
					])
				]);
			}

			// Generate checksum
			$paymentData['checksum'] = $bayarcash->createPaymentIntenChecksumValue(
				$option['secret_key'],
				$paymentData
			);

			// Create payment intent
			$response = $bayarcash->createPaymentIntent($paymentData);

			if (!isset($response->url)) {
				throw new \Exception(__('Invalid response from payment gateway', 'bayarcash-for-paymattic'));
			}

			// Update transaction
			$transaction_model = new Transaction();
			$transaction_model->updateTransaction($transaction->id, [
				'payment_mode' => $option['sandbox_mode'] ? 'test' : 'live',
				'charge_id' => $response->id ?? ''
			]);

			// Log payment initiation with fee information
			$this->log_payment_initiation_with_fee($form->ID, $submission->id, $payment_method, $response->url, $gateway_fee);

			// Send success response
			wp_send_json_success([
				'message' => __('You are redirecting to Bayarcash to complete the purchase. Please wait while you are redirecting....', 'bayarcash-for-paymattic'),
				'call_next_method' => 'normalRedirect',
				'redirect_url' => $response->url
			], 200);

		} catch (\Exception $e) {
			wp_send_json_error([
				'message' => $e->getMessage()
			], 400);
		}
	}

	private function log_payment_initiation_with_fee($form_id, $submission_id, $payment_method, $redirect_url, $gateway_fee) {
		$fee_message = $gateway_fee > 0 ? sprintf(" Gateway Fee: RM %.2f.", $gateway_fee) : "";

		do_action('wppayform_log_data', [
			'form_id' => $form_id,
			'submission_id' => $submission_id,
			'type' => 'info',
			'created_by' => 'Bayarcash Payment',
			'title' => 'Payment Initiated',
			'content' => sprintf(
				"Payment initiated with Bayarcash. Payment Method: %s (ID: %d).%s Redirecting to: %s",
				$this->payment_methods[$payment_method]['title'],
				$this->payment_methods[$payment_method]['id'],
				$fee_message,
				$redirect_url
			)
		]);
	}

	public function callback() {
		if (!isset($_POST['order_number'])) {
			return;
		}

		$this->process_payment_response($_POST, true);
		wp_send_json_success();
	}

	public function redirect($data) {
		if (!isset($data['gateway_return'])) {
			return;
		}

		$submission_id = absint($data['wppayform_payment']);
		$payment_method = $data['payment_method'];
		$submission_hash = $data['submission_hash'];

		// Process payment first
		$this->process_payment_response($data, false);

		// Get submission and redirect
		$submission = (new Submission())->getSubmission($submission_id);
		if ($submission) {
			$redirect_url = $this->getSuccessURL(Form::getForm($submission->form_id), $submission);
			wp_redirect($redirect_url);
			exit;
		}
	}

	private function process_payment_response($data, $is_callback = false): void {
		// Get submission ID from either POST or GET
		$submission_id = $is_callback
			? (isset($data['order_number']) ? absint($data['order_number']) : 0)
			: (isset($data['wppayform_payment']) ? absint($data['wppayform_payment']) : 0);

		if (!$submission_id) {
			return;
		}

		// Get submission and transaction
		$submission = (new Submission())->getSubmission($submission_id);
		$transaction = $this->getTransaction( $submission_id );

		if (!$transaction || !$submission) {
			return;
		}

		// Check if payment is already processed
		if ($transaction->status === 'paid') {
			return;
		}

		// Get payment data from either POST or GET
		$payment_data = $this->normalize_payment_data($data, $is_callback);
		$status = isset($payment_data['status']) ? absint($payment_data['status']) : 0;

		// Prepare log content
		$logContent = sprintf(
			"%s | Bank: %s | Amount: %s %s | Transaction ID: %s | Reference: %s | Status Description: %s | Datetime: %s",
			$status === 3 ? "Payment completed successfully" : "Payment failed",
			$payment_data['payer_bank_name'] ?? 'N/A',
			$payment_data['currency'] ?? 'N/A',
			$payment_data['amount'] ?? 'N/A',
			$payment_data['transaction_id'] ?? 'N/A',
			$payment_data['exchange_reference_number'] ?? 'N/A',
			$payment_data['status_description'] ?? 'N/A',
			$payment_data['datetime'] ?? current_time('Y-m-d H:i:s')
		);

		// Process payment status
		if ($status === 3) {
			do_action('wppayform_log_data', [
				'form_id' => $submission->form_id,
				'submission_id' => $submission_id,
				'type' => 'success',
				'created_by' => 'Bayarcash Payment',
				'title' => 'Payment Status Updated',
				'content' => $logContent
			]);

			$this->handlePaid($submission, $transaction, $payment_data);
		} elseif ($status > 0) {
			do_action('wppayform_log_data', [
				'form_id' => $submission->form_id,
				'submission_id' => $submission_id,
				'type' => 'error',
				'created_by' => 'Bayarcash Payment',
				'title' => 'Payment Status Updated',
				'content' => $logContent
			]);

			$this->handleFailed($submission, $transaction, $payment_data);
		}

	}

	private function normalize_payment_data($data, $is_callback) {
		if ($is_callback) {
			return $data; // POST data is already in the correct format
		}

		// Convert GET parameters to match POST format
		return [
			'transaction_id' => sanitize_text_field($data['transaction_id'] ?? ''),
			'exchange_reference_number' => sanitize_text_field($data['exchange_reference_number'] ?? ''),
			'exchange_transaction_id' => sanitize_text_field($data['exchange_transaction_id'] ?? ''),
			'order_number' => sanitize_text_field($data['order_number'] ?? ''),
			'currency' => sanitize_text_field($data['currency'] ?? ''),
			'amount' => sanitize_text_field($data['amount'] ?? ''),
			'payer_bank_name' => sanitize_text_field($data['payer_bank_name'] ?? ''),
			'status' => sanitize_text_field($data['status'] ?? ''),
			'status_description' => sanitize_text_field($data['status_description'] ?? ''),
			'checksum' => sanitize_text_field($data['checksum'] ?? ''),
			'datetime' => current_time('Y-m-d H:i:s')
		];
	}

	private function handlePaid($submission, $transaction, $payment_data) {
		$updateData = [
			'status' => 'paid',
			'payment_note' => maybe_serialize($payment_data),
			'updated_at' => current_time('Y-m-d H:i:s')
		];

		(new Transaction())->updateTransaction($transaction->id, $updateData);
		(new Submission())->where('id', $submission->id)->update([
			'payment_status' => 'paid',
			'updated_at' => current_time('Y-m-d H:i:s')
		]);

		do_action('wppayform/form_payment_success', $submission, $transaction, $transaction->form_id, $updateData);
	}

	private function handleFailed($submission, $transaction, $payment_data) {
		$updateData = [
			'status' => 'failed',
			'payment_note' => maybe_serialize($payment_data),
			'updated_at' => current_time('Y-m-d H:i:s')
		];

		(new Transaction())->updateTransaction($transaction->id, $updateData);
		(new Submission())->where('id', $submission->id)->update([
			'payment_status' => 'failed',
			'updated_at' => current_time('Y-m-d H:i:s')
		]);
	}

	private function initialize_bayarcash($option): Bayarcash {
		$bayarcash = new Bayarcash(trim($option['personal_access_token']));

		$bayarcash->setApiVersion('v3');

		if ($option['sandbox_mode']) {
			$bayarcash->useSandbox();
		}

		return $bayarcash;
	}

	private function log_payment_initiation($form_id, $submission_id, $payment_method, $redirect_url) {
		do_action('wppayform_log_data', [
			'form_id' => $form_id,
			'submission_id' => $submission_id,
			'type' => 'info',
			'created_by' => 'Bayarcash Payment',
			'title' => 'Payment Initiated',
			'content' => sprintf(
				"Payment initiated with Bayarcash. Payment Method: %s (ID: %d). Redirecting to: %s",
				$this->payment_methods[$payment_method]['title'],
				$this->payment_methods[$payment_method]['id'],
				$redirect_url
			)
		]);
	}

	private function determine_payment_method($form_data) {
		return Arr::get($form_data, '__wpf_selected_payment_method');
	}

	private function is_form_currency_supported($currency) {
		if (!in_array($currency, $this->supported_currencies)) {
			wp_send_json_error([
				'message' => sprintf(
					__('Currency not supported. Only MYR is supported. Current: %s', 'bayarcash-for-paymattic'),
					$currency
				)
			], 400);
		}
	}

	private function get_settings($form_id): array {
		$options = get_option(PAYMATTIC_BAYARCASH_SLUG);
		$postfix = '';

		if (!empty($options['form-customize-' . $form_id])) {
			$postfix = "-$form_id";
		}

		return [
			'sandbox_mode' => !empty($options['enable-sandbox' . $postfix]),
			'personal_access_token' => trim($options['personal-access-token' . $postfix]),
			'portal_key' => trim($options['portal-key' . $postfix]),
			'secret_key' => trim($options['api-secret-key' . $postfix])
		];
	}

	private function getTransaction( $value ) {
		return (new Transaction())->where( 'submission_id', $value)->first();
	}

	private function getSuccessURL($form, $submission) {
		$confirmation = Form::getConfirmationSettings($form->ID);
		$confirmation = ConfirmationHelper::parseConfirmation($confirmation, $submission);

		// Get the transaction to determine the payment method
		$transaction = $this->getTransaction( $submission->id );
		$payment_method = $transaction ? $transaction->payment_method : 'bayarcash';

		if (($confirmation['redirectTo'] == 'customUrl' && $confirmation['customUrl']) ||
		    ($confirmation['redirectTo'] == 'customPage' && $confirmation['customPage'])) {

			$url = $confirmation['redirectTo'] == 'customUrl'
				? $confirmation['customUrl']
				: get_permalink(intval($confirmation['customPage']));

			return PlaceholderParser::parse(
				add_query_arg(['payment_method' => $payment_method], $url),
				$submission
			);
		}

		$globalSettings = get_option('wppayform_confirmation_pages');
		if (isset($globalSettings['confirmation']) && $globalSettings['confirmation']) {
			return add_query_arg([
				'wpf_submission' => $submission->submission_hash,
				'payment_method' => $payment_method
			], get_permalink(intval($globalSettings['confirmation'])));
		}

		return add_query_arg([
			'wpf_submission' => $submission->submission_hash,
			'payment_method' => $payment_method
		], home_url());
	}

	public function validate_subscription($payment_items, $formatted_elements, $form_data, $subscription_items) {
		wp_send_json_error([
			'message' => __('Bayarcash doesn\'t support subscriptions at this time', 'bayarcash-for-paymattic'),
			'payment_error' => true
		], 423);
	}

	public function add_transaction_url($transactions, $submission_id) {
		foreach ($transactions as $transaction) {
			if ($transaction->charge_id) {
				$base_url = $transaction->payment_mode === 'test'
					? 'https://sandbox.bayar.cash/transactions/'
					: 'https://gateway.bayar.cash/transactions/';

				$transaction->transaction_url = $base_url . $transaction->charge_id;
			}
		}
		return $transactions;
	}

	private function get_customer_name($entries) {
		$name_fields = ['customer_name', 'name', 'full_name'];
		foreach ($name_fields as $field) {
			if (isset($entries[$field]) && !empty($entries[$field]['value'])) {
				return substr($entries[$field]['value'], 0, 128);
			}
		}
		return '';
	}

	private function get_customer_email($entries) {
		$email_fields = ['customer_email', 'email'];
		foreach ($email_fields as $field) {
			if (isset($entries[$field]) && !empty($entries[$field]['value'])) {
				return $entries[$field]['value'];
			}
		}
		return '';
	}

	private function get_customer_phone($entries) {
		$phone_fields = ['customer_phone', 'phone', 'mobile', 'telephone'];
		foreach ($phone_fields as $field) {
			if (isset($entries[$field]) && !empty($entries[$field]['value'])) {
				return $entries[$field]['value'];
			}
		}
		return '';
	}
}